from .imports import*


def load_data_CSV(data:str, target:str, cut: Union[str, list] = None) -> Tabular: # Returns tabular data
    '''A function that creates a omnixai.data.tabular.Tabular object instance representing a particular dataset.
    
    :param str data: Pathname for the CSV file where the dataset is found.
    :param str target: Target variable used for training data
    :param Union[str, list] cut: Variables that should be ignored in training. By default, None.
    :return: Tabular object instance representing 'data'
    :rtype: Tabular
    :raises ValueError: if data cannot be loaded
    '''
    try:
        df = pd.read_csv(data) #We read the dataset onto a dataframes object
        if cut is not None:
          df.drop(cut, axis = 1, index = None, columns = None, level = None, inplace = True, errors = 'raise') #Remove columns the user doesn't want to include
          print(df)
        tabular_data = Tabular(df, target_column=target) #Create a Tabular object (needed for future training and explaining) and specify the target column. Omnixai does not allow multiple targets, so datasets containing 2 or more targets need to be passed twice through the program, with one of the targets being cut. 
        return tabular_data #Return data through a Tabular object
    except:
        raise ValueError("Unable to load data properly. Make sure your file is in the same directory and that the target is present") #This means that there was a problem reading the data, cutting columns, or creating the Tabular object

def load_data_sklearn(datastore:dict, target:str, cut: Union[str, list] = None) -> Tabular: # Returns tabular data
    '''A function that creates a omnixai.data.tabular.Tabular object instance representing a particular sklearn dataset for demoing.
 
    :param dict datastore: A dictionary object containing the data
    :param str target: Target variable used for training data
    :param Union[str, list] cut: Variables that should be ignored in training. By default, None
    :return: Tabular object instance representing 'data'
    :rtype: Tabular
    :raises ValueError: if data cannot be loaded
    '''
    try:
        df = pd.DataFrame(data=datastore.data, columns=datastore.feature_names) #We read the dataset (exluding target) onto a dataframes object
        df["target"] = datastore.target #We add the target column
        if cut is not None:
          df.drop([cut], axis = 1, index = None, columns = None, level = None, inplace = True, errors = 'raise') #Remove columns the user doesn't want to include
        tabular_data = Tabular(df, target_column=target) #Create a Tabular object (needed for future training and explaining) and specify the target column. Omnixai does not allow multiple targets, so datasets containing 2 or more targets need to be passed twice through the program, with one of the targets being cut. 
        return tabular_data #Return data through a Tabular object
    except:
        raise ValueError("Unable to load data properly. Make sure your file is in the same directory and that the target is present") #This means that there was a problem reading the data, cutting columns, or creating the Tabular object




from .imports import*

def compare_explanations(filenames:list, verbose = False): #Analyze the generated explanations in given files
  '''A function that analyzes and compares the explanations generated by train_and_explainModel.
    
    :param list filenames: File names with explanations (of the form "Explainer ImportanceScores - Model Target.csv")
    :return: None
    '''
  explainers = []
  features = [] #All files should have same features
  for filename in filenames:
      try: 
        df = pd.read_csv(filename)

        for i in range(len(df['features'])):
            if(verbose):
                print (df['features'][i])
            df['features'][i] = ast.literal_eval(df['features'][i])
            #df.loc[:, ('features', i)] = ast.literal_eval(df['features'][i])

        for i in range(len(df['scores'])):
            if(verbose):
                print (df['features'][i])
            df['scores'][i] = ast.literal_eval(df['scores'][i])
            #df.loc[:, ('scores', i)] = ast.literal_eval(df['scores'][i])

        features = df['features']
        scores = df['scores']
        explainer = filename.split()[0]
        explainers.append(explainer)

        for feature in features[0]:
            vars()[feature + explainer + "List"] = []
            for i in range(len(df['features'])):
                eval(feature + explainer + "List").append(scores[i][features[i].index(feature)])

        vars()[explainer + "maxScore"] = []
        for score in scores:
            eval(explainer + "maxScore").append(max(score))   
      except Exception as e:
        print("An error occurred while analyzing the graph. " + str(e))

  for feature in features[0]:
      for explainer in explainers:
          plt.plot(eval(feature + explainer + "List"))
          plt.xlabel("Instance #")
          plt.ylabel("Importance Score")
      plt.title("Change in importance of " + feature + " over instance number " + "- " + ' '.join([str(elem) for elem in explainers]))
      plt.show()

  for explainer in explainers:
      plt.plot(eval(explainer + "maxScore"))
      plt.xlabel("Instance #")
      plt.ylabel("Importance Score")
  plt.title("Change in importance of most important feature over instance number " + "- " + ' '.join([str(elem) for elem in explainers]))
  plt.show()  
  

  for feature in features[0]:
    feature_explainer_lists = []
    for explainer in explainers:
        feature_explainer_lists.append(eval(feature + explainer + "List"))
    correlation = np.corrcoef([x for x in feature_explainer_lists])
    if(len(explainers) == 2):
        print ("Correlation between " + ' and '.join([str(elem) for elem in explainers]) + " for feature " + feature + ": " + str(correlation[1,0]))
    else:
        print (correlation)



from setuptools import setup, find_namespace_packages

setup(name='XAISuite',
      version='0.6.6',
      description='XAISuite: Training and Explanation Generation Utilities for Machine Learning Models',
      long_description=open("README.md", "r", encoding="utf-8").read(),
      long_description_content_type="text/markdown",
      keywords="XAI Explainable AI Explanation Machine Learning Models",
      url="https://github.com/11301858/XAISuite",
      author = "Shreyan Mitra",
      install_requires=[
        "omnixai>=1.2"
        "dash"
        "dash_bootstrap_components"
      ],
       packages=["xaisuite", "demo"],
      )



#Needed Libraries

#Data Storage, Export, Access, and Visualization
import pandas as pd
import csv
from pathlib import Path
import matplotlib.pyplot as plt
import ast

#Machine Learning Models and Training
from sklearn.model_selection import train_test_split

acceptedModels = {"SVC": "sklearn.svm.SVC", "NuSVC": "sklearn.svm.NuSVC", "LinearSVC": "sklearn.svm.LinearSVC", "SVR": "sklearn.svm.SVR", "NuSVR": "sklearn.svm.SVR", "LinearSVR": "sklearn.svm.LinearSVR", 
                 "AdaBoostClassifier": "sklearn.ensemble.AdaBoostClassifier", "AdaBoostRegressor": "sklearn.ensemble.AdaBoostRegressor", "BaggingClassifier": "sklearn.ensemble.BaggingClassifier", "BaggingRegressor": "sklearn.ensemble.BaggingRegressor",
                 "ExtraTreesClassifier": "sklearn.ensemble.ExtraTreesClassifier", "ExtraTreesRegressor": "sklearn.ensemble.ExtraTreesRegressor", 
                 "GradientBoostingClassifier": "sklearn.ensemble.GradientBoostingClassifier", "GradientBoostingRegressor": "sklearn.ensemble.GradientBoostingRegressor",
                 "RandomForestClassifier": "sklearn.ensemble.RandomForestClassifier", "RandomForestRegressor": "sklearn.ensemble.RandomForestRegressor",
                 "StackingClassifier": "sklearn.ensemble.StackingClassifier", "StackingRegressor": "sklearn.ensemble.StackingRegressor",
                 "VotingClassifier": "sklearn.ensemble.VotingClassifier", "VotingRegressor": "sklearn.ensemble.VotingRegressor",
                 "HistGradientBoostingClassifier": "sklearn.ensemble.HistGradientBoostingClassifier", "HistGradientBoostingRegressor": "sklearn.ensemble.HistGradientBoostingRegressor",
                 "GaussianProcessClassifier": "sklearn.gaussian_process.GaussianProcessClassifier", "GaussianProcessRegressor": "sklearn.gaussian_process.GaussianProcessRegressor", 
                 "IsotonicRegression": "sklearn.isotonic.IsotonicRegression", "KernelRidge": "sklearn.kernel_ridge.KernelRidge", 
                 "LogisticRegression": "sklearn.linear_model.LogisticRegression", "LogisticRegressionCV": "sklearn.linear_model.LogisticRegressionCV",
                 "PassiveAgressiveClassifier": "sklearn.linear_model.PassiveAggressiveClassifier", "Perceptron": "sklearn.linear_model.Perceptron", 
                  "RidgeClassifier": "sklearn.linear_model.RidgeClassifier", "RidgeClassifierCV": "sklearn.linear_model.RidgeClassifierCV",
                  "SGDClassifier": "sklearn.linear_model.SGDClassifier", "SGDOneClassSVM": "sklearn.linear_model.SGDOneClassSVM", 
                  "LinearRegression": "sklearn.linear_model.LinearRegression", "Ridge": "sklearn.linear_model.Ridge", 
                  "RidgeCV": "sklearn.linear_model.RidgeCV", "SGDRegressor": "sklearn.linear_model.SGDRegressor",
                  "ElasticNet": "sklearn.linear_model.ElasticNet", "ElasticNetCV": "sklearn.linear_model.ElasticNetCV",
                  "Lars": "sklearn.linear_model.Lars", "LarsCV": "sklearn.linear_model.LarsCV", 
                  "Lasso": "sklearn.linear_model.Lasso", "LassoCV": "sklearn.linear_model.LassoCV",
                  "LassoLars": "sklearn.linear_model.LassoLars", "LassoLarsCV": "sklearn.linear_model.LassoLarsCV",
                  "LassoLarsIC": "sklearn.linear_model.LassoLarsIC", "OrthogonalMatchingPursuit": "sklearn.linear_model.OrthogonalMatchingPursuit",
                  "OrthogonalMatchingPursuitCV": "sklearn.linear_model.OrthogonalMatchingPursuitCV", "ARDRegression": "sklearn.linear_model.ARDRegression",
                  "BayesianRidge": "sklearn.linear_model.BayesianRidge", "MultiTaskElasticNet": "sklearn.linear_model.MultiTaskElasticNet", 
                  "MultiTaskElasticNetCV": "sklearn.linear_model.MultiTaskElasticNetCV", "MultiTaskLasso": "sklearn.linear_model.MultiTaskLasso",
                  "MultiTaskLassoCV": "sklearn.linear_model.MultiTaskLassoCV", "HuberRegressor": "sklearn.linear_model.HuberRegressor",
                  "QuantileRegressor": "sklearn.linear_model.QuantileRegressor", "RANSACRegressor": "sklearn.linear_model.RANSACRegressor",
                  "TheilSenRegressor": "sklearn.linear_model.TheilSenRegressor", "PoissonRegressor": "sklearn.linear_model.PoissonRegressor",
                  "TweedieRegressor": "sklearn.linear_model.TweedieRegressor", "GammaRegressor": "sklearn.linear_model.GammaRegressor", 
                  "PassiveAggressiveRegressor": "sklearn.linear_model.PassiveAggressiveRegressor", "BayesianGaussianMixture": "sklearn.mixture.BayesianGaussianMixture",
                  "GaussianMixture": "sklearn.mixture.GaussianMixture", 
                  "OneVsOneClassifier": "sklearn.multiclass.OneVsOneClassifier", "OneVsRestClassifier": "sklearn.multiclass.OneVsRestClassifier", 
                  "OutputCodeClassifier": "sklearn.multiclass.OutputCodeClassifier", "ClassifierChain": "sklearn.multioutput.ClassifierChain", 
                   "RegressorChain": "sklearn.multioutput.RegressorChain",  "MultiOutputRegressor": "sklearn.multioutput.MultiOutputRegressor",
                   "MultiOutputClassifier": "sklearn.multioutput.MultiOutputClassifier", "BernoulliNB": "sklearn.naive_bayes.BernoulliNB", 
                  "CategoricalNB": "sklearn.naive_bayes.CategoricalNB", "ComplementNB": "sklearn.naive_bayes.ComplementNB", 
                  "GaussianNB": "sklearn.naive_bayes.GaussianNB", "MultinomialNB": "sklearn.naive_bayes.MultinomialNB", 
                  "KNeighborsClassifier": "sklearn.neighbors.KNeighborsClassifier", "KNeighborsRegressor": "sklearn.neighbors.KNeighborsRegressor", 
                  "BernoulliRBM": "sklearn.neural_network.BernoulliRBM", "MLPClassifier": "sklearn.neural_network.MLPClassifier", "MLPRegressor": "sklearn.neural_network.MLPRegressor",  
                  "DecisionTreeClassifier": "sklearn.tree.DecisionTreeClassifier", "DecisionTreeRegressor": "sklearn.tree.DecisionTreeRegressor",
                  "ExtraTreeClassifier": "sklearn.tree.ExtraTreeClassifier", "ExtraTreeRegressor": "sklearn.tree.ExtraTreeRegressor"
                 }

from sklearn.base import is_classifier, is_regressor

#OmniXAI Explanatory Models and Visualization
from omnixai.data.tabular import Tabular
from sklearn.preprocessing import*
from omnixai.preprocessing.base import Identity
from omnixai.preprocessing.tabular import TabularTransform
from omnixai.explainers.tabular import TabularExplainer
from omnixai.visualization.dashboard import Dashboard

#For documentation
from typing import Union
